import dnd, es, spe, popuplib, gamethread, playerlib, cmdlib
from dnd import dnd
from spe import HookAction
from spe import HookType

def load():
    spe.parseINI('dnd/subscripts/signatures/mysigs.ini')
    spe.parseTypesINI('dnd/subscripts/signatures/mytypes.ini')
    spe.detourFunction('Respawn', HookType.Pre, pre_spawn)
    cmdlib.registerClientCommand('!slow', slow, 'Paladin ability - slows enemy based on % hp')
    info = popuplib.create("Paladin Info")
    info.addline('Paladin Class Information:')
    info.addline('  Lay On Hands - When brought below 20% Health,')
    info.addline('    you are instantly healed. 1/round')
    info.addline('  Level 3: Divine Grace - Your armor gives Damage Reduction')
    info.addline('    and regenerates')
    info.addline(' Level 5: Divine Health - Increased Health')
    info.addline(' Level 9: Holy Sword - Your bullets deal magic damage')
    info.addline('-> 9. Main Menu')
    info.submenu(9, 'main_menu')
    
def unload():
    popuplib.delete('Paladin Info')
    spe.undetourFunction('Respawn', HookType.Pre, pre_spawn)
    cmdlib.unregisterClientCommand('!slow')
    
def distance(user1, user2):
    x,y,z = es.getplayerlocation(user1)
    x1,y1,z1 = es.getplayerlocation(user2)
    return math.sqrt(math.pow((x-x1), 2) + math.pow((y-y1), 2) + math.pow((z-z1), 2))
    
def pre_spawn(args):
    userid = es.getuserid(es.gethandlefromindex(spe.getEntityIndex(args[0])))
    if dnd.Player(userid).getClass() == 'Paladin':
        if dnd.Temp(userid).get('paladin respawn') == 1:
            dnd.Temp(userid).set('paladin respawn', 0)
            dnd.Temp(userid).set('health', 50)
        else:
            if dnd.Player(userid).getLevel() >= 20:
                dnd.Temp(userid).set('paladin respawn', 1)
        dnd.Temp(userid).add('Fortitude', dnd.Player(userid).getLevel()/2+2)
        dnd.Temp(userid).add('Reflex', dnd.Player(userid).getLevel()/2+2)
        dnd.Temp(userid).add('Will', dnd.Player(userid).getLevel()/2+2)
        dnd.Temp(userid).add('Lay On Hands', 1)
        if dnd.Player(userid).getLevel() >= 5 and dnd.Player(userid).getLevel() < 20:
            dnd.Temp(userid).add('health', min(dnd.Player(userid).getLevel() * 5, 50))
    return (HookAction.Continue, 0)
        
def round_start(ev):
    armor_loop()
    damage_loop()
    heal_loop()
    slow_loop()
    for userid in es.getUseridList():
        if dnd.Player(userid).getClass() == 'Paladin':
                if dnd.Player(userid).getLevel() >= 3:
                    dnd.armorregen(ev['userid'], 3, 5)
                    
def armor_loop():
    for userid in es.getUseridList():
        if dnd.Player(userid).getClass() == 'Paladin' and not es.getplayerprop(userid, 'CBasePlayer.pl.deadflag'):
            if dnd.Player(userid).getLevel() >= 3:
                dnd.Temp(userid).set('damage reduction', int(playerlib.getPlayer(userid).getArmor())/2)
    gamethread.delayedname(.1, 'dndtick', armor_loop)
    
def damage_loop():
    for userid in es.getUseridList():
        if dnd.Player(userid).getClass() == 'Paladin' and not es.getplayerprop(userid, 'CBasePlayer.pl.deadflag'):
            if dnd.Player(userid).getLevel() >= 11:
                for player in es.getUseridList():
                    if not es.getplayerprop(player, 'CBasePlayer.pl.deadflag') and distance(userid, player) < 600 and es.getplayerteam(player) != es.getplayerteam(userid):
                        dnd.damage(player, userid, 1, 'True')
    gamethread.delayedname(3, 'dndtick', damage_loop)
    
def heal_loop():
    for userid in es.getUseridList():
        if dnd.Player(userid).getClass() == 'Paladin' and not es.getplayerprop(userid, 'CBasePlayer.pl.deadflag'):
            if dnd.Player(userid).getLevel() >= 13:
                for player in es.getUseridList():
                    if not es.getplayerprop(player, 'CBasePlayer.pl.deadflag') and distance(userid, player) < 600 and es.getplayerteam(player) == es.getplayerteam(userid):
                        dnd.heal(player, 3)
    gamethread.delayedname(2, 'dndtick', heal_loop)
    
def slow(userid, args):
    if dnd.Player(userid).getClass() == 'Paladin':
        if dnd.Player(userid).getLevel() >= 17:
            if dnd.Temp(userid).get('aoe pally slow'):
                dnd.dndTell(userid, 'Your slow is already enabled')
            else:
                dnd.dndTell(userid, 'Your slow has been enabled')
                dnd.Temp(userid).set('aoe pally slow', 1)
                gamethread.delayedname(dnd.Player(userid).getLevel() - 10, slowOff, userid)
                
def slowOff(userid):
    dnd.Temp(userid).set('aoe pally slow', 0)
    dnd.dndTell(userid, 'Your slow has been deactivated')
    
    
def slow_loop():
    for userid in es.getUseridList():
        if dnd.Player(userid).getClass() == 'Paladin' and not es.getplayerprop(userid, 'CBasePlayer.pl.deadflag'):
            if dnd.Temp(userid).get('aoe pally slow'):
                for player in es.getUseridList():
                    if not es.getplayerprop(player, 'CBasePlayer.pl.deadflag') and distance(userid, player) < 600 and es.getplayerteam(player) != es.getplayerteam(userid):
                        slow_percent = -.3 / (float(es.getplayerprop(userid, 'CBasePlayer.m_iHealth')) / float(dnd.Temp(userid).get('health')))
                        dnd.speed(player, slow_percent, set=None, delay=1)
    gamethread.delayedname(1, 'dndtick', slow_loop)
    
def player_death(ev):
    userid = ev['userid']
    if dnd.Player(userid).getClass() == 'Paladin' and dnd.Temp(userid).get('paladin respawn'):
        spe.respawn(userid)
